% BEZIER_CURVE - compute geometry and derivatives of it using a third order
%   Bezier curve formula.
%
% Parameters:
%          svec - the parameter,  s \in [0,1] (can be a row vector with
%                 several parameters, then we compute results for all of
%                 and return them also as row vectors)
%   x1,x2,x3,x4 - the four control points for the Bezier curve as column
%                 vectors
%
% Returns:
%              x - the location in geometry
%             xs - the first derivative of that location with respect to s
%            xss - the second derivative of that location with respect to s
%
% File        : bezier.m
% Date        : 2009-03-13
% Author      : Oliver Pajonk

% Rollercoaster Prototype - A MATLAB rollercoaster
% Copyright (C) 2009  Oliver Pajonk
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function [x, xs, xss] = bezier_curve(svec, x1, x2, x3, x4)    
    x = zeros(length(x1),length(svec));
    if nargout > 1
        xs = zeros(length(x1),length(svec));
    end
    if nargout > 2
        xss = zeros(length(x1),length(svec));
    end
    
    for i = 1:length(svec)
        s = svec(i);
        x(:,i) = (1-s).^3 .* x1 + 3.*(1-s).^2.*s .* x2 + 3.*(1-s).*s.^2 .* x3 + s.^3 .* x4;
        if nargout > 1
            xs(:,i) = 3.*x2.*(1-s).^2 - 6.*x2.*(1-s).*s + 6.*x3.*(1-s) .*s - 3.*x3.*s.^2 - 3.*(1-s).^2.*x1 + 3.*s.^2.*x4;
        end
        if nargout > 2
            xss(:,i) = -12 .*x2.*(1-s) + 6.*x3.*(1-s) + 6.*x2.*s - 12.*x3.*s + 6.*(1-s).*x1 + 6.*s.*x4;
        end
    end
end