% EULER_FORWARD - Euler forward method to integrate ODEs
%   The interface is compatible with the MATLAB ODE solvers.
%
% Parameters:
%           ode - the ODE to integrate
%             s - the previous value
%          delt - step size
% 
% Returns:
%         snext - the approximation of the ODE solution for t+delt
%
% File        : euler_forward.m
% Date        : 2009-03-18
% Author      : Oliver Pajonk

% Rollercoaster Prototype - A MATLAB rollercoaster
% Copyright (C) 2009  Oliver Pajonk
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function [ T, s ] = euler_forward(ode, T, s0)
    s = zeros(2, length(s0));
    step = 1;
    s(step,:) = s0;
    while (step < length(T))
        step = step + 1;
        delt = T(step) - T(step-1);
        s(step,:) = euler_forward_step(ode, s(step-1,:)', delt);
    end
end

function [ snext ] = euler_forward_step(ode, s, delt)
    snext = s + delt*ode(0, s);
end

