% ROLLERCOASTER_ODE - compute the rollercoaster ODE on the given geometry
%
% Parameters:
%             s - the location parameter s
%            ds - time derivative of the location parameter
%       geomery - a function to compute the geometry at parameter s
%      varargin - optional parameters to the geometry
%
% Returns:
%           dds - the second time derivative of the location parameter
%
% File        : rollercoaster_ode.m
% Date        : 2009-03-13
% Author      : Oliver Pajonk

% Rollercoaster Prototype - A MATLAB rollercoaster
% Copyright (C) 2009  Oliver Pajonk
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function [ dds ] = rollercoaster_ode(s, ds, geometry, varargin)
    [x, xs, xss] = geometry(s, varargin{:});
    
    % gravity in 3D space
    g = [0; 0; 9.81];

    dds = -1 * (xs'*g + ds^2*xs'*xss)/(xs'*xs);
end

