% ROLLERCOASTER_ODE_SYSTEM - compute the rollercoaster ODE system
%
% Parameters:
%         s - the initial values
%   v, a, b - all control points for the closed Bezier curve as column
%                 vectors
%
% Returns:
%      sdot - the solution of the ODE system
%
% File        : rollercoaster_ode_system.m
% Date        : 2009-03-13
% Author      : Oliver Pajonk

% Rollercoaster Prototype - A MATLAB rollercoaster
% Copyright (C) 2009  Oliver Pajonk
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function [sdot] = rollercoaster_ode_system(s, v, a, b)
    % Now we finally need to know the geometry to deduce the correct
    % parameters to the rollercoaster ODE
    [dims nodes] = size(v);
    sdot = zeros(2,1);
    node = mod(floor(s(1)), nodes) + 1;
    if (node ~= nodes)
        x1 = v(:,node);
        x2 = a(:,node);
        x3 = b(:,node);
        x4 = v(:,node+1);
    else
        x1 = v(:,node);
        x2 = a(:,node);
        x3 = b(:,node);
        x4 = v(:,1);
    end
    sdot(1) = s(2);
    sdot(2) = rollercoaster_ode(s(1) - floor(s(1)), s(2), @bezier_curve, x1, x2, x3, x4);
end



